/*************************************************************************

         name: folTestSuite.pl
      version: July 27, 2001
  description: Test Suite with Propositional Formulas for Provers 
      authors: Patrick Blackburn & Johan Bos
 
*************************************************************************/

:- module(folTestSuite,[formula/2]).


/*========================================================================
   Formulas
========================================================================*/

formula(all(X,or(p(X),not(p(X)))),'Theorem').

formula(some(X,or(p(X),not(p(X)))),'Theorem').

formula(imp(all(X,p(X)),not(some(Y,not(p(Y))))),'Theorem').

formula(imp(all(X,imp(p(X),q(X))),imp(all(X,p(X)),all(X,q(X)))),'Theorem').

formula(imp(imp(p,all(X,q(X))),all(X,imp(p,q(X)))),'Theorem').

formula(some(X,or(p(X),q(X))),'Non-theorem, Satisfiable').

% Schubert's Steamroller
%
% Wolves, foxes, birds, caterpillars, and snails are animals, and
% there are some of each of them. Also there are some grains, and
% grains are plants. Every animal either likes to eat all plants
% or all animals much smaller than itself that like to eat some
% plants. Caterpillars and snails are much smaller than birds,
% which are much smaller than foxes, which in turn are much
% smaller than wolves. Wolves do not like to eat foxes or grains,
% while birds like to eat caterpillars but not snails.
% Caterpillars and snails like to eat some plants. Therefore
% there is an animal that likes to eat a grain eating animal.

formula(imp(and(all(X,imp(wolf(X),animal(X))),
            and(all(X,imp(fox(X),animal(X))),
            and(all(X,imp(bird(X),animal(X))),
            and(all(X,imp(caterpillar(X),animal(X))),
            and(all(X,imp(snail(X),animal(X))),
            and(all(X,imp(grain(X),plant(X))),
            and(some(Y,wolf(Y)),
            and(some(Y,fox(Y)),
            and(some(Y,bird(Y)),
            and(some(Y,caterpillar(Y)),
            and(some(Y,snail(Y)),
            and(some(Y,grain(Y)),
            and(all(X,imp(animal(X),or(all(Y,imp(plant(Y),eats(X,Y))),
                    all(Y1,imp(and(animal(Y1),and(much_smaller(Y1,X),
                        some(Z,and(plant(Z),eats(Y1,Z))))),eats(X,Y1)))))),
            and(all(X,all(Y,imp(and(bird(Y),or(snail(X),caterpillar(X))),
                much_smaller(X,Y)))),
            and(all(X,all(Y,imp(and(bird(X),fox(Y)),much_smaller(X,Y)))),
            and(all(X,all(Y,imp(and(fox(X),wolf(Y)),much_smaller(X,Y)))),
            and(all(X,all(Y,imp(and(wolf(X),or(fox(X),grain(Y))),not(eats(X,Y))))),
            and(all(X,all(Y,imp(and(bird(X),caterpillar(Y)),eats(X,Y)))),
            and(all(X,all(Y,imp(and(bird(X),snail(Y)),not(eats(X,Y))))),
                all(X,imp(or(caterpillar(X),snail(X)),
                      some(Y,and(plant(Y),eats(X,Y)))))))))))))))))))))))),
            some(X,some(Y,and(animal(X),and(animal(Y),some(Z,and(
                 grain(Z),and(eats(Y,Z),eats(X,Y))))))))),'Theorem').


formula(imp(and(all(X,imp(wolf(X),animal(X))),
            and(all(X,imp(fox(X),animal(X))),
            and(all(X,imp(bird(X),animal(X))),
            and(all(X,imp(caterpillar(X),animal(X))),
            and(all(X,imp(snail(X),animal(X))),
            and(all(X,imp(grain(X),plant(X))),
            and(some(Y,wolf(Y)),
            and(some(Y,fox(Y)),
            and(some(Y,bird(Y)),
            and(some(Y,caterpillar(Y)),
            and(some(Y,snail(Y)),
            and(some(Y,grain(Y)),
            and(not(some(X,much_smaller(X,X))),  % added
            and(all(X,all(Y,all(Z,imp(and(much_smaller(X,Y),much_smaller(Y,Z)),much_smaller(X,Z))))), % added
            and(not(some(X,eats(X,X))),          % added   
            and(not(some(X,and(animal(X),plant(X)))), %added
            and(all(X,imp(animal(X),or(all(Y,imp(plant(Y),eats(X,Y))),
                    all(Y1,imp(and(animal(Y1),and(much_smaller(Y1,X),
                        some(Z,and(plant(Z),eats(Y1,Z))))),eats(X,Y1)))))),
            and(all(X,all(Y,imp(and(bird(Y),or(snail(X),caterpillar(X))),
                much_smaller(X,Y)))),
            and(all(X,all(Y,imp(and(bird(X),fox(Y)),much_smaller(X,Y)))),
            and(all(X,all(Y,imp(and(fox(X),wolf(Y)),much_smaller(X,Y)))),
            and(all(X,all(Y,imp(and(wolf(X),or(fox(X),grain(Y))),not(eats(X,Y))))),
            and(all(X,all(Y,imp(and(bird(X),caterpillar(Y)),eats(X,Y)))),
            and(all(X,all(Y,imp(and(bird(X),snail(Y)),not(eats(X,Y))))),
                all(X,imp(or(caterpillar(X),snail(X)),
                      some(Y,and(plant(Y),eats(X,Y)))))))))))))))))))))))))))),
            some(X,some(Y,and(animal(X),and(animal(Y),some(Z,and(
                 grain(Z),and(eats(Y,Z),eats(X,Y))))))))),'Theorem').


formula(some(X,and(p(X),not(p(X)))),'Non-theorem, Unsatisfiable').

formula(some(X,and(man(X),some(Y,woman(Y)))),'Not a theorem, Satisfiable').

formula(some(X,and(man(X),all(Y,imp(woman(Y),love(X,Y))))),'Not a theorem, Satisfiable').

formula(all(Y,imp(woman(Y),some(X,and(man(X),love(X,Y))))),'Not a theorem, Satisfiable').


